/*************************************************************************
 *  Desktop.h include file
 *  Copyright(c) 1998 Jan Curn. All rights reserved.
 *
 *   This file contains the desktop functions.
 *
 ************************************************************************/

#ifndef DESKTOP__H
#define DESKTOP__H

#ifndef __WIN32__
#error Desktop routines can be used only in a Win32 Application.
#endif

#ifndef __cplusplus
#error Must use C++ for the Desktop routines.
#endif

#include <Windows.h>
#include <ShlObj.h>

/* Tato funkce vytvori link na soubor 'lpszPathObj', tento link ulozi
   do souboru 'lpszPathLink'. Parametr 'lpszDesc' obsahuje popis linku,
   muze to byt libovolny text.

   Funkce vola funkci CoCreateInstance, takze predpoklada, ze funkce
   CoInitialize jiz byla zavolana. CreateLink pouziva rozhrani IPersistFile
   pro ulozeni linku a rozhrani IShellLink pro ulozeni jeho jmena a popisu.

   Funkce vraci promennou typu HRESULT, pokud tedy chcete zjistit, jak
   uspela, pouzijte makro SUCCEEDED(hres).                */

inline HRESULT CreateLink(LPSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc)
{
    // Promenne
    HRESULT hres;
    IShellLink* psl;

    // Zjistime ukazatel na rozhrani IShellLink.
    hres = CoCreateInstance(CLSID_ShellLink, NULL,
        CLSCTX_INPROC_SERVER, IID_IShellLink, (void **)&psl);

    if(SUCCEEDED(hres)) // Podarilo se ?
    {
        // Promenna
        IPersistFile* ppf;

        // Ulozime cestu k objektu na ktery si prejeme link
        psl->SetPath(lpszPathObj);

        // Ulozime popis objektu
        psl->SetDescription(lpszDesc);

        // Zjistime ukazatel na rozhrani IPersistFile
        hres = psl->QueryInterface(IID_IPersistFile, (void **)&ppf);

        if (SUCCEEDED(hres)) // Podarilo se ?
        {
            // Unicode retezec
            wchar_t wsz[MAX_PATH];

            // Prelozime retezec z ANSI do UNICODE
            MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1, wsz, MAX_PATH);

            // A konecne ulozime link volanim IPersistFile::Save.
            hres = ppf->Save(wsz, TRUE);

            // Uvolnime objekt
            ppf->Release();
        }
        psl->Release();
    }
    return hres; // Konec funkce
}

#endif  /* DESKTOP__H */


