/*************************************************************************
 *  DirEx.h include file
 *  Copyright(c) 1998 Jan Curn. All rights reserved.
 *
 *   This file contains the directory functions.
 *
 ************************************************************************/

#ifndef DIREX__H
#define DIREX__H

#ifndef __WIN32__
#error Directory routines can be used only in a Win32 Application.
#endif

#ifndef __cplusplus
#error Must use C++ for the Directory routines.
#endif

#include <Windows.h>
#include <Except.h>

/* Tato funkce vytvori adresar lpPathName, i kdyz jeho nadadresar(e) jeste
   neexistuje(i). Parametr lpSecuirityAttributes specifikuje
   bezpecnostni atributy nove vzniklych adresaru. Pokud je NULL,
   ma implicitni bezpecnost. Pokud funkce uspeje, vraci TRUE, pokud ne,
   vraci FALSE. Funkci GetLastError() muzete ziskat vice informaci o chybe.
   Pozor, funkce podporuje pouze ANSI. */

BOOL CreateSubDirectory(LPCSTR lpPathName, LPSECURITY_ATTRIBUTES lpSecurityAttributes)
{
  // Promenne
  DWORD     dwCountOfChars = 0; // Pocitadlo znaku '\\'
  CHAR      lpNormPathName[MAXPATH]; // Cesta v normalizvanem tvaru
  CHAR      **lpMatrix;         // Matice s tabulkou subadresaru
  CHAR      *lpAddress;         // Pomocna adresa
  HANDLE    hFindFile;          // Handle na nalezeny soubor
  WIN32_FIND_DATA FD;           // Struktura s informacemi o nalezenem souboru

  // Inicializace retezce
  ZeroMemory(lpNormPathName, sizeof(lpNormPathName));

  // Kontroly
  if(!lpPathName) // Neni parametr nahodou NULL ???
  {
    SetLastError(ERROR_BAD_PATHNAME);
    return(FALSE);
  }

  // Neni cesta kratsi nez 4 znaky, nebo delsi nez MAXPATH-2
  if(strlen(lpPathName) <= 3 || strlen(lpPathName) >= (MAXPATH - 2))
  {
    SetLastError(ERROR_BAD_PATHNAME);
    return(FALSE);
  }

  // Normalizujeme cestu do tvaru "(X:)\\...\\..."
  if(lpPathName[1]!=':' && lpPathName[0]!='\\')
  {
    // Udelame si misto pro backslash
    strcpy(lpNormPathName+1, lpPathName);
    lpNormPathName[0] = '\\';
  }
  else
    strcpy(lpNormPathName, lpPathName);

  // Jeste oriznem pripadny backslash na konci
  if(lpNormPathName[strlen(lpNormPathName)-1] == '\\')
    lpNormPathName[strlen(lpNormPathName)-1] = NULL;

  // Spocitame, kolik backslashu je v retezci dohromady
  for(DWORD i=0; i<strlen(lpNormPathName); i++)
    if(lpNormPathName[i]=='\\')
      dwCountOfChars++;

  // Alokace matrixu o rozmerech MAXPATH * dwCountOfChars
  try
  {
    lpMatrix = new CHAR*[dwCountOfChars];
    for (DWORD i=0; i<dwCountOfChars; i++)
      lpMatrix[i] = new CHAR[MAXPATH];
  }
  catch(xalloc)
  {
    SetLastError(ERROR_NOT_ENOUGH_MEMORY);
    return(FALSE);
  }

  // Vytvorime tabulku subadresaru a ulozime ji do matrixu
  for(DWORD i=0; i<dwCountOfChars; i++)
  {
    strcpy(lpMatrix[i], lpNormPathName);
    // Orezavani podadresaru
    for(DWORD j=0; j<i; j++)
    {
      lpAddress = strrchr(lpMatrix[i], '\\');
      lpAddress[0] = NULL;
    }
  }

  // A konecne adresare vytvorime - v obracenem poradi, nez jsou v tabulce
  for(INT i=dwCountOfChars-1; i>=0; i--)
  {
    // Zkusime nalezt dany adresar
    hFindFile = FindFirstFile(lpMatrix[i], &FD);

    // Pokud byl nalezen, ale neni to adresar, tak chyba
    if(hFindFile != INVALID_HANDLE_VALUE &&
       hFindFile != 0                    &&
       !(FD.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
    {
      SetLastError(ERROR_BAD_PATHNAME);
      return(FALSE);
    }

    // Pokud nebyl nalezen, tak ho vytvorime
    if(hFindFile==0 || hFindFile==INVALID_HANDLE_VALUE)
      if(!CreateDirectory(lpMatrix[i], lpSecurityAttributes))
        return(FALSE); // Nepodarilo se vytvorit adresar, konec...

    // Zavreme handle
    FindClose(hFindFile);
  }

  // Dealokace matice
  for(DWORD i=0; i<dwCountOfChars; i++)
    delete[] lpMatrix[i];
  delete[] lpMatrix;

  SetLastError(ERROR_SUCCESS);
  return(TRUE); // Vse OK
}

#endif  /* DIREX__H */


