/*************************************************************************
 *  QJokes.h include file
 *  Copyright(c) 1998 Jan Curn, All rights reserved.
 *
 *  Trida Jokes.
 *  Poznamky: Tato trida pracuje pouze v ANSI kodovani.
 *            Trida pouziva generator nahodnych cisel.
 *
 ************************************************************************/

#ifndef QJOKES__H
#define QJOKES__H

#include <String.h>
#include <StdLib.h>
#include <Stdio.h>
#include "QMain.h"

#ifndef __cplusplus
#error Must use C++ for the type Jokes.
#endif

#ifndef WIN32
#error QJokes.h can only be used in a Win32 Aplication.
#endif

typedef class QJokes{
  private:
    HANDLE  hFile;          // Handle na soubor 'Vtipy.dat'
    HANDLE  hMapping;       // Handle na pametove mapovanej soubor
    CHAR    *lpData;        // Ukazatel dat pametovyho mapovani
    DWORD   dwNumOfJokes;   // Pocet vtipu
    DWORD   dwCurrentJoke;  // Obsahuje cislo nahodne aktualniho vtipu  
  public:
    QJokes();               /* Konstruktor - pouze vynuluje promenne,
                            ale neuvolnuje prostredky. */
    CHAR    lpBuffer[128];  /* Pomocny buffer - pri chybe jakekoliv metody
                            obsahuje text s presnym znenim chyby. */
    BOOL    Initialize(CHAR *lpFileName); /* Inicializace tridy, lpFileName
                            musi obsahovat jmeno ANSI souboru s vtipy.
                            Metoda vraci vysledek operace (TRUE/FALSE). */
    DWORD   GetNumOfJokes();/* Vrati pocet vtipu v prave
                            otevrenem souboru, nebo 0 pri chybe. */
    BOOL    GetJoke(CHAR *lpString, DWORD dwSize, DWORD dwNum); /* Do lpString
                            ulozi vtip z aktualne otevreneho souboru
                            s indexem dwNum. Velikost lpString (vcetne
                            NULL) je ulozena v dwSize. Metoda vraci
                            vysledek operace (TRUE/FALSE). Pokud nastala
                            nejaka chyba, obsah lpString neni zmenen. */
    BOOL    GetRandomJoke(CHAR *lpString, DWORD dwSize); /* Stejne jako
                            GetJoke(), ale tentokrat se do lpString ulozi
                            vtip s nahodnym indexem. */
    void    Release();      /* Tato metoda uvolni obsah cele tridy (zavre
                            soubory, mapovani, ...). Mela by byt volana,
                            kdyz uz se trida nebude pouzivat. */
} Jokes, *lpJokes;

//
// *** Deklarace metod
//
inline       QJokes::QJokes()           { hFile = hMapping = NULL; lpData = NULL; dwNumOfJokes = 0; dwCurrentJoke = 0; memset(lpBuffer, 0, sizeof(lpBuffer)); }
inline DWORD QJokes::GetNumOfJokes()    { if(lpData && hFile && hFile!=INVALID_HANDLE_VALUE) return(dwNumOfJokes); else return(0); }
inline void  QJokes::Release()          { if(lpData) UnmapViewOfFile(lpData); if(hMapping) CloseHandle(hMapping); if(hFile && hFile!=INVALID_HANDLE_VALUE) CloseHandle(hFile); QJokes(); }
inline BOOL  QJokes::GetRandomJoke(CHAR *lpString, DWORD dwSize)    { dwCurrentJoke++; if(dwCurrentJoke >= dwNumOfJokes) dwCurrentJoke = 0; GetJoke(lpString, dwSize, random(3) ? dwCurrentJoke : random(dwNumOfJokes)); }

inline BOOL QJokes::Initialize(CHAR *lpFileName)
{
  // Pomocne promenne
  WORD      EnterChar = '\r\n'; // Znak ENTERu
  CHAR      *lpAddr;            // Pomocna adresa
  LONG      lToEnd;             // Kolik byte zbyva do konce souboru

  // Uvolneni puvodniho obsahu
  Release();

  // Otevreni souboru
  hFile = CreateFileA(lpFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, NULL);
  if(!hFile || hFile==INVALID_HANDLE_VALUE)
  {
    sprintf(lpBuffer, "Error (1): Could not open the file %s !", strrchr(lpFileName, '\\') ? strrchr(lpFileName, '\\')+1 : lpFileName);
    return(FALSE);
  }

  // Test velikosti souboru
  if(GetFileSize(hFile, NULL)>=0x100000 || GetFileSize(hFile, NULL)<0x10)
  {
    CloseHandle(hFile);
    sprintf(lpBuffer, "Error (2): The size of the file must be in the range of 10B to 1MB !");
    return(FALSE);
  }

  // Otevreni pametovyho mapovani
  hMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, GetFileSize(hFile, NULL), NULL);
  if(!hMapping)
  {
    CloseHandle(hFile);
    sprintf(lpBuffer, "Error (3): Could not create the mapping of the file %s !", strrchr(lpFileName, '\\') ? strrchr(lpFileName, '\\')+1 : lpFileName);
    return(FALSE);
  }

  // Mapovani obrazu v pametove mapovanym souboru
  lpData = (CHAR *)MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, GetFileSize(hFile, NULL));
  if(!lpData)
  {
    CloseHandle(hFile);
    CloseHandle(hMapping);
    sprintf(lpBuffer, "Error (4): Could not read the file %s !", strrchr(lpFileName, '\\') ? strrchr(lpFileName, '\\')+1 : lpFileName);
    return(FALSE);
  }

  // Pocitani vtipu (resp. ENTERu)
  dwNumOfJokes = 1;  // Soubor prece nekonci ENTERem.
  lpAddr = lpData;
  do
  {
    lToEnd = (LONG)GetFileSize(hFile, NULL) - (LONG)(lpAddr - lpData);
    if(lToEnd <= 0)
      break;
    lpAddr = (CHAR *)FindData(lpAddr, &EnterChar, (DWORD)lToEnd, sizeof(EnterChar));
    if(lpAddr) // Nalezli jsme ENTER (resp. dalsi vtip)
    {
      dwNumOfJokes++;
      lpAddr = lpAddr + 2; // Inkrementace o dva byte (znaky '\r\n')
    }
  }
  while (lpAddr);

  // Inicializace MCRNG
  randomize();

  // Nasledujici upravy jsou pro ziskani objektivne nahodneho cisla
  dwCurrentJoke = (ABS(ABS(rand()) - 1983) + GetTickCount()) % dwNumOfJokes;

  sprintf(lpBuffer, "Error (0): No error, %u jokes successfully loaded.", dwNumOfJokes);
  return(TRUE); // Vse OK
}

inline BOOL QJokes::GetJoke(CHAR *lpString, DWORD dwSize, DWORD dwNum)
{
  // Pomocne promenne
  WORD      EnterChar = '\r\n'; // Znak ENTERu
  CHAR      *lpAddr;            // Pomocna adresa
  LONG      lToEnd;             // Kolik byte zbyva do konce
  DWORD     dwJokesFound;       // Index posledne nalezenho vtipu
  DWORD     dwSizeOfJoke;       // Velikost vtipu

  if(dwNum >= dwNumOfJokes)
  {
    sprintf(lpBuffer, "Error (5): The parameter 'dwNum' in call to 'GetJoke()' must be less than 'GetNumOfJokes()'!");
    return(FALSE);
  }

  if(!lpData || !hFile || hFile==INVALID_HANDLE_VALUE)
  {
    sprintf(lpBuffer, "Error (6): The class 'Jokes' is not initialized !");
    return(FALSE);
  }

  // Hledani pozadovaneho vtipu
  lpAddr = lpData;
  dwJokesFound = 0;
  dwSizeOfJoke = 0;
  do
  {
    lToEnd = (LONG)GetFileSize(hFile, NULL) - (LONG)(lpAddr - lpData);
    if(lToEnd <= 0)
      { lpAddr = NULL; break; }
    if(dwJokesFound>=dwNum) // Nasli jsme pozadovany vtip,
    {                       // nyni zjistime jeho delku
      CHAR *lpHelpAddr; // Pomocna adresa
      lpHelpAddr = (CHAR *)FindData(lpAddr, &EnterChar, lToEnd, sizeof(EnterChar));
      if(lpHelpAddr)
        dwSizeOfJoke = lpHelpAddr - lpAddr;
      else
        dwSizeOfJoke = lToEnd; // Vtip je na konci souboru
      break;
    }
    lpAddr = (CHAR *)FindData(lpAddr, &EnterChar, lToEnd, sizeof(EnterChar));
    if(lpAddr) // Nasli jsme ENTER (resp. dalsi vtip)
    {
      lpAddr = lpAddr + 2; // Inkrementace o dva byte (znaky '\r\n')
      dwJokesFound++;      // Inkrementace o jedna
    }
  }
  while (lpAddr);

  if(!dwSizeOfJoke || !lpAddr)
  {
    sprintf(lpBuffer, "Error (7): Could not find the joke 'dwNum' !");
    return(FALSE);
  }

  // Finalni kopirovani vtipu do retezce
  strncpy(lpString, lpAddr, MIN(dwSizeOfJoke, dwSize-1));
  lpString[MIN(dwSizeOfJoke, dwSize-1)] = NULL;
  sprintf(lpBuffer, "Error (0): The joke successfully read.");
  return(TRUE); // Vse OK */
}

#endif /* QJOKES__H */
