/*************************************************************************
 *  QMain.h include file
 *  Copyright(c) 1998 Jan Curn. All rights reserved.
 *
 *   This include file contains many useful things.
 *
 ************************************************************************/

#ifndef __QMAIN_H
#define __QMAIN_H

#ifdef __cplusplus
extern "C" {
#endif

#if ((defined(_WIN32) || defined(__WIN32) || defined(__WIN32__) || defined(__FLAT__)) && !defined(WIN32))
#define WIN32 __WIN32__
#endif

#if (defined(WIN32) && !defined(__FLAT__))
#define __FLAT__ WIN32
#endif

#if defined(UNICODE) || defined(_UNICODE)
#define UNICODE
#define _UNICODE
#define TCHAR	_TCHAR
#endif

#include <_null.h>
#include <tchar.h>

/*** Makra **************************************************************/
#define COPYRIGHT	_TEXT("Copyright(c) 1998 Jan Curn.\n\r")

#ifndef AND
  #define AND		&
  #define OR		|
  #define XOR		^
#endif

#ifndef _SIZE_T
#define _SIZE_T
     typedef unsigned size_t;
#endif

#ifndef TRUE
  #define TRUE			1
  #define FALSE			0
#endif

/* Makra pro alokaci normalniho pole a matice-like pole, ne pomoci
   operatorou new, ale pomoci funkce Win32 HeapAlloc().
   Popis: NORM_ALLOC  - Makro alokuje pamet na halde udane handlem HEAP o
                        velikosti NUM objektu TYPE do ukazatele POINTER,
                        pro vsechny objekty je volan konstruktor s parametry
                        C (vcetne zavorek).
          MAT_ALLOC   - Vse stejne jako vyse, ovsem toto makro alokuje
                        pseudo-matici o velikosti NUM_X a NUM_Y.
          ..._ACCESS  - Makro zajisti pristup k pseudomatici nebo normalnimu
                        poli. Parametry INDEX (INDEX_X, INDEX_Y) udavaji
                        pozici.
          ..._DEALLOC - Uvolni pamet alokovanou makrem ..._ALLOC,
                        toto makro vola normalne destruktory.
          ..._SIZE    - Vraci velikost pameti, ktera se alokovala
 Poznamky: -C verze maker nevolaji konstruktory, ale oblast pouze vynuluji.
           -Pokud jako parametr HEAP uvedete hodnotu 0 (NULL), pouzije se
            pro alokaci implicitni halda procesu.                */
#ifdef WIN32
#ifdef __cplusplus // C++ verze volaji konstruktor s parametry C
#define NORM_ALLOC(POINTER, TYPE, HEAP, NUM, C)           { if(NUM) { if(HEAP) POINTER = (TYPE *)HeapAlloc((HEAP), HEAP_ZERO_MEMORY,(NUM)*sizeof(TYPE)); else POINTER = (TYPE *)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,(NUM)*sizeof(TYPE)); } else POINTER = NULL; if(POINTER) for(l_int ALLOC_##POINTER=0; ALLOC_##POINTER<(NUM); ALLOC_##POINTER++) { POINTER[ALLOC_##POINTER] = TYPE##C; } }
#else              // C verze oblast pouze vynuluje
#define NORM_ALLOC(POINTER, TYPE, HEAP, NUM, C)           { if(NUM) { if(HEAP) POINTER = (TYPE *)HeapAlloc((HEAP), HEAP_ZERO_MEMORY,(NUM)*sizeof(TYPE)); else POINTER = (TYPE *)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,(NUM)*sizeof(TYPE)); } else POINTER = NULL; }
#endif  // __cplusplus
#define NORM_ACCESS(POINTER, INDEX)                       POINTER[INDEX]
#define NORM_SIZE(POINTER, HEAP)                          ((HEAP)?(HeapSize((HEAP), 0, (LPCVOID)(POINTER))):(HeapSize(GetProcessHeap(), 0, (LPCVOID)(POINTER))))
#define NORM_DEALLOC(POINTER, HEAP, TYPE)                 { if(POINTER) { int size = NORM_SIZE(POINTER, HEAP); for(l_int i=0; i<(size/sizeof(POINTER[0])); i++) POINTER[i].~TYPE(); if(HEAP) HeapFree((HEAP),0,(LPVOID)(POINTER)); else HeapFree(GetProcessHeap(),0,(LPVOID)(POINTER)); } POINTER = NULL; }
#define MAT_ALLOC(POINTER, TYPE, HEAP, NUM_X, NUM_Y, C)   NORM_ALLOC(POINTER, TYPE, HEAP, ((NUM_X)*(NUM_Y)*sizeof(TYPE)), C);
#define MAT_ACCESS(POINTER, INDEX_X, INDEX_Y, NUM_X)      POINTER[((SIZE_X)*(Y))+(X)]
#define MAT_SIZE(POINTER, HEAP)                           NORM_SIZE(POINTER, HEAP)
#define MAT_DEALLOC(POINTER, HEAP, TYPE)                  NORM_DEALLOC(POINTER, HEAP, TYPE)
#endif // WIN32

// Pomocna makra k praci s maticemi-like poli bez pouziti heap
#define M_SIZE(WIDTH_X, HEIGHT_Y, SIZE)                   ((WIDTH_X)*(HEIGHT_Y)*(SIZE))
#define M_ACCESS(PTR, X, Y, WIDTH_X)                      PTR[(WIDTH_X)*(Y) + (X)]

// Typedefy
typedef unsigned char		u_char; /*  %u    8        0-255            */
#ifdef WIN32
typedef unsigned int		l_int;  /*  %u   32        0-4 294 967 295  */
#else
typedef unsigned long int	l_int;  /*  %u   32        0-4 294 967 295  */
#endif
typedef unsigned short int	s_int;  /*  %u   16        0-65 535         */
typedef short int			n_int;  /*  %d   16   -32735-32735          */
typedef double				uhel;   /*  %lf - uhel v RAD				*/
typedef u_char				U_CHAR; /*  %u    8        0-255            */
typedef l_int				L_INT;  /*  %u   32        0-4 294 967 295  */
typedef s_int				S_INT;  /*  %u   16        0-65 536         */
typedef n_int				N_INT;  /*  %d   16   -32735-32735          */
typedef uhel				UHEL;   /*  %lf - uhel v RAD                */

// Tato funkce najde blok lpData2 v bloku lpData1, velikost bloku lpData1
// je dwSize1, lpData2 dwSize2. Vraci se ukazatel na tuto oblast, nebo
// NULL pri nenalezeni/chybe.
inline LPVOID FindData(LPVOID lpData1, LPVOID lpData2, DWORD dwSize1, DWORD dwSize2)
{
  LONG  lToEnd;                     // Kolik byte zbyva do konce dat 1
  BYTE *lpPtr  = (BYTE *)lpData1;   // Pomocny ukazatel
  if(!lpData1 || !lpData2 || !dwSize1 || !dwSize2 || dwSize2>dwSize1 || dwSize1>=0x7FFFFFFEL)
    return(NULL);                   // Chyba
  do                                // Hlavni smycka
  {
    lToEnd = dwSize1 - (lpPtr - (BYTE *)lpData1);
    if(lToEnd<=0 || lToEnd<(LONG)dwSize2) // Kontrola
      return(NULL);
    lpPtr = (BYTE *)memchr(lpPtr, (int)((BYTE *)lpData2)[0], (size_t)lToEnd);
    if(lpPtr)                       // Nalezli jsme prvni znak,
    {                               // tak overime i ostatni.
      lToEnd = dwSize1 - (lpPtr - (BYTE *)lpData1);
      if(lToEnd<=0 || lToEnd<(LONG)dwSize2)
	return(NULL);
      if(!memcmp(lpPtr, lpData2, dwSize2))
	return((LPVOID)lpPtr);          // Nasli jsme to !!!
      lpPtr++;                      // Inkremenatce ukazatele
    }
  } while (lpPtr);                  // Dokud se neco nachazi
  return(NULL);                     // Nic se nenaslo...
}

// Matematicka makra a funkce
#ifndef MAX
#  define MAX(x,y)	((x) > (y) ? (x) : (y))
#  define MIN(x,y)	((x) < (y) ? (x) : (y))
#endif

#define ROZDIL(one,two)			((one)>(two)?((one)-(two)):((two)-(one)))

#define Q_PI	3.14159265358979

#ifndef PI
#define PI		Q_PI
#endif

#ifndef M_PI
#define M_PI    Q_PI
#endif

#define Q_ERR_ANGLE		100000.0  /* Takovyto uhel je de facto blbost,
                                   a proto slouzi k identifikaci chyby */

/* Makra prevedou dekadicky uhel na radialni a obracene
   Pro uhly pouzivejte pouze typ double ! */
#define Q_RAD(DEG_)		((DEG_) * 0.01745329252) /* PI/180 */
#define Q_DEG(RAD_)		((RAD_) * 57.295779513)  /* 180/PI */
#ifndef RAD
#define RAD		Q_RAD
#define DEG		Q_DEG
#endif

#define RAD0	0.0
#define RAD10   0.1745329251994
#define RAD15   0.2617993877991
#define RAD20   0.3490658503989
#define RAD30   0.5235987755983
#define RAD40   0.6981317007977
#define RAD45   0.7853981633974
#define RAD50   0.8726646259972
#define RAD60   1.047197551197
#define RAD70   1.221730476396
#define RAD80   1.396263401595
#define RAD90	1.570796326795
#define RAD120  2.094395102393
#define RAD180  3.141592653589 // = -RAD180
#define RAD270  4.712388980385 // = -RAD90
#define RAD360  6.28318530718  // = RAD0 = 2PI

#define ANGLE(ANG)	fmod(ANG, RAD360) // Zkrati uhel

#define Q_ABS(NUM)      ((NUM) < 0 ? (-NUM) : (NUM))
#ifndef ABS
#  define ABS           Q_ABS
#endif

#define MOCNINA2(NUM)	((NUM) * (NUM))
#define MOCNINA3(NUM)	((NUM) * (NUM) * (NUM))
#define MOCNINA4(NUM)	((NUM) * (NUM) * (NUM) * (NUM))

/* Vypocita plochu ctverce, kde N1 a N2 jsou jeho krajni body */
#define CTVEREC(N1, N2)		(((N1)>(N2)) ? ((N1)-(N2))*((N1)-(N2)) : ((N2)-(N1))*((N2)-(N1)))
#define PYTHAG(A, B)		sqrt((double)((A)*(A)+(B)*(B)))

/* Vypocita vzdalenost dvou bodu na rovine (resp. v prostoru) */
#define VZDALENOST(A1, B1, A2, B2)		  		    sqrt((double)(CTVEREC(A1,A2) + CTVEREC(B1, B2)))
#define VZDALENOST2D                                VZDALENOST
#define VZDALENOST3D(X1, Y1, Z1, X2, Y2, Z2)		sqrt((double)(CTVEREC(X1,X2) + CTVEREC(Y1,Y2) + CTVEREC(Z1,Z2)))

/*
 * Format COLOR16:
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |   Red   |   Green   |  Blue   |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * Poznamky: Lidske oko je nejcitlivejsi na zelenou barvu, tak je ulozena
 *           s nejvetsi presnosti.
 */
typedef	unsigned short int		BARVA16, *PBARVA16, COLOR16, *PCOLOR16;
typedef	BARVA16					barva16, color16, *pbarva16, *pcolor16;
typedef	BARVA16					Barva16, Color16, *pBarva16, *pColor16;

#define COLOR16_GET_R(RGB)		(((RGB) >> 8) & 0xFF)
#define COLOR16_GET_G(RGB)		(((RGB) >> 3) & 0xFF)
#define COLOR16_GET_B(RGB)		(((RGB) << 3) & 0xFF)
#define COLOR16_SET_R(RGB,R)    ((((R) & 0x00F8) << 8) | ((RGB) & 0x07FF))
#define COLOR16_SET_G(RGB,G)    ((((G) & 0x00FC) << 3) | ((RGB) & 0xF81F))
#define COLOR16_SET_B(RGB,B)    ((((B) & 0x00F8) >> 3) | ((RGB) & 0xFFE0))
#define COLOR16_GET_I(RGB)      (COLOR16_GET_R(RGB)/3 + COLOR16_GET_G(RGB)*5/9 + COLOR16_GET_B(RGB)/9)
#define COLOR16_SET_I(RGB,I)    ( /* ...Doplnit... */ )
#define COLOR16_MAKE(R,G,B)     ((((R) & 0x00F8) << 8) | (((G) & 0x00FC) << 3) | (((B) & 0x00F8) >> 3))
#define COLOR16_CREATE(C1, C2)  COLOR16_MAKE(((COLOR16_GET_R(C1)+COLOR16_GET_R(C2))/2),((COLOR16_GET_G(C1)+COLOR16_GET_G(C2))/2),((COLOR16_GET_B(C1)+COLOR16_GET_B(C2))/2))

#define BARVA16_GET_R           COLOR16_GET_R
#define BARVA16_GET_G           COLOR16_GET_G
#define BARVA16_GET_B           COLOR16_GET_B
#define BARVA16_SET_R           COLOR16_SET_R
#define BARVA16_SET_G           COLOR16_SET_G
#define BARVA16_SET_B           COLOR16_SET_B
#define BARVA16_GET_I           COLOR16_GET_I
#define BARVA16_SET_I           COLOR16_SET_I
#define BARVA16_MAKE            COLOR16_MAKE
#define BARVA16_CREATE          COLOR16_CREATE
/* Typ COLOR16 slouzi k ulozeni barvy v 16bitove skale (65536 barev).Je to
   normalni prejmenovany typ WORD. K praci s nim slouzi vyse uvedena makra.
   Priklad pouziti:  unsigned char b_R = 128;
                     COLOR16 col = COLOR16_MAKE(28, 231, b_R);
                     col = COLOR16_SET_R(barv, 16);
   Vracene hodnoty:  COLOR16_GET_R(barv).............16
                     COLOR16_GET_G(barv)............231
                     COLOR16_GET_B(barv)............128
*/

#define BILA16			((COLOR16)0xFFFF)
#define CERNA16			((COLOR16)0x0000)
#define ZLUTA16         ((COLOR16)0xFFE0)
#define CERVENA16       ((COLOR16)0xF800)
#define ZELENA16        ((COLOR16)0x07E0)
#define MODRA16         ((COLOR16)0x001F)
#define NEUTRALNI16		((COLOR16)0xFC1F) // Pokud je barva neutralni,
                                          // vykresluje se barva pozadi.
#define WHITE16  		BILA16
#define BLACK16         CERNA16
#define YELLOW16        ZLUTA16
#define RED16           CERVENA16
#define GREEN16         ZELENA16
#define BLUE16          MODRA16
#define NEUTRAL16       NEUTRALNI16
/************************************************************************/
/************************************************************************/
/************************************************************************/

// Makro pro zjisteni velikosti pole
#define chDIMOF(Array)	(sizeof(Array)/sizeof(Array[0]))

// Toto makro zobrazi Message Box
#ifdef WIN32
#define chMB(s) { _TCHAR szTMP[128];                \
GetModuleFileName(NULL, szTMP, chDIMOF(szTMP));     \
MessageBox(GetActiveWindow(), s, szTMP, MB_OK); }
#endif

// Varovani pri pouziti Unicode
#if defined(UNICODE) && defined(WIN32)
#define chWARNIFUNICODEUNDERWIN95() \
if(GetWindowsDirectoryW(NULL, 0) <= 0)  \
  MessageBoxA(NULL, "This operating system doesn't support Unicode.", NULL, MB_OK);
#else
#define chWARNIFUNICODEUNDERWIN95();
#endif

typedef unsigned (__stdcall *PTHREAD_START) (void *);
#define chBEGINTHREADEX(lpsa, cbStack, lpStartAddr, lpvThreadParam, fdwCreate, lpIDThread) \
        ((HANDLE)_beginthreadex((void *)(lpsa), (unsigned)(cbStack), (PTHREAD_START)(lpStartAddr), (void *)(lpvThreadParm), (unsigned)(fdwCreate), (unsigned *)(lpIDThread)))

#define chHANDLE_DLGMSG(hwnd, message, fn) \
        case (message): return (SetDlgMsgResult(hwnd, uMsg, HANDLE_##message((hwnd), (wParam), (lParam), (fn))))

#define chINITSTRUCT(structure, fInitSize) \
        (ZeroMemory(&(structure), sizeof(structure)), fInitSize ? (*(int*) &(structure) = sizeof(structure)) : 0)

#define chSETDLGICONS(hwnd, idiLarge, idiSmall)                         \
{                                                                       \
  OSVERSIONINFO VerInfo;                                                \
  chINITSTRUCT(VerInfo, TRUE);                                          \
  GetVersionEx(&VerInfo);                                               \
  if(VerInfo.dwPlatformId==VER_PLATFORM_WIN32_NT && VerInfo.dwMajorVersion<=3 && VerInfo.dwMinorVersion<=51)        \
  {                                                                     \
    SetClassLong(hwnd, GCL_HICON, (LONG)LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiSmall)));              \
  }                                                                     \
  else                                                                  \
  {                                                                     \
    SendMessage(hwnd, WM_SETICON, TRUE, (LPARAM)LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiLarge)));      \
    SendMessage(hwnd, WM_SETICON, FALSE, (LPARAM)LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiSmall)));     \
  }                                                                     \
}

// Pouziti: #pragma chMsg(Tohle jeste dodelej)
#define chSTR(x)    #x
#define chSTR2(x)   chSTR(x)
#define chMSG(desc) message(__FILE__"("chSTR2(__LINE__)"):" #desc)

#ifdef __cplusplus
}
#endif

#endif /* __QMAIN_H */




