/*************************************************************************
 *  Uninst.h include file
 *  Copyright(c) 1998 Jan Curn. All rights reserved.
 *
 *   Tento soubor obsahuje univerzalni odinstalacni funkce.
 *
 ************************************************************************/

#ifndef UNINST__H
#define UNINST__H

#ifdef __DLL__
#error Uninstall routines can be used only in an EXE File.
#endif

#ifndef __WIN32__
#error Uninstall routines can be used only in a Win32 Application.
#endif

#include <dos.h>       // int _argc, char **_argv
#include <string.h>    // strcpy(), ...

#ifndef MAXPATH
#define MAXPATH     260
#endif

#ifndef BIGSIZE
#define BIGSIZE     520
#endif


#ifdef __cplusplus
extern "C" {
#endif

/* Tato funkce zjistuje, zda se ma provest odinstalovani. Kontroluje
   totiz, zda je v parametrech v prikazove radce uveden prepinac
   'lpUninstallString'. Na velikost pismen se nebere ohled. */
BOOL WINAPI TestForUninstall(CHAR *lpUninstallString)
{
  // Prohlidnuti vsech parametru
  for(DWORD i=0; i<(DWORD)_argc; i++)
    if(!strcmpi(lpUninstallString, _argv[i]))
      return(TRUE);
  return(FALSE);
}

/* Tato funkce provede samotne odinstalovani, to se sklada z techto fazi:
     1)  Vymazani souboru 'lpFiles', 'dwNumOfFiles' udava pocet techto
         souboru. Retezec nesmi zacinat znakem '\'.
     2)  Odstraneni klicu 'lpRegKeys' z registru, 'dwNumOfKeys' udava
         jejich pocet.
     3)  Odstraneni hodnot 'lpRegValues' z registru, 'dwNumOfValues'
         udava opet jejich pocet.
   Pozor: Jak 'lpRegKeys' tak 'lpRegValues' musi obsahovat jmeno
          zakladniho klice, priklad spravneho zadani klice je:
          "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Run"
   Pozor: Neni mozne mazat klice, ktere obsahuji dalsi podklice, proto
          je potreba, aby jednotlive podklice byly uvedeny v matici
          'lpRegKeys' take, a to na nizsich indexech, aby byli odstraneny drive.
   Pozor: Dale neni mozno mazat hodnoty a klice, ktere patri primo
          nekteremu ze zakladnich klicu (HKEY_LOCAL_MACHINE, ...) .
   Parametr 'lpDirectory' slouzi k predani zakladniho adresare, od ktereho
   se bude relativne urcovat cesta k souborum 'lpFiles'. Pokud je tento
   parametr NULL, pouzije se startovni adresar programu, pozor, ne aktualni !
   Parametr 'lpRoutine' slouzi k predani adresy ukoncovaci rutiny programu,
   ta musi do cca 3 sekund ukoncit cely program, aby bylo mozne smazat jeho
   zdrojovy soubor. Rutina musi mit nasledujici prototyp:
                     VOID WINAPI ExitRoutine(VOID);
   Pokud rutina program neukonci, samotny program a pripadne jeho dynamicke
   knihovny nebudou smazany. Pri spravnem prubehu funkce vysledek nevraci
   (protoze dojde k ukonceni programu), pokud ale nastala chyba, funkce
   vraci tyto hodnoty:
        0.........Ukoncovaci rutina neukoncila program, mohlo jiz dojit
                  k vymazani nekterych souboru.
        1.........Nedostatek mista na disku pro nahrani mazaciho programu
        2.........Nedostatek pameti, hodnoty a klice jiz byly odstraneny.
        3.........Nelze nainstalovat mazaci program, hodnoty a klice
                  jiz byly odstraneny.
        4.........Parametr 'lpDirectory' je chybny, hodnoty a klice byly
                  jiz odstraneny.                                                   */

DWORD WINAPI Uninstall(CHAR  *lpFiles[MAXPATH],
               CHAR  *lpRegKeys[MAXPATH],
               CHAR  *lpRegValues[MAXPATH],
               DWORD dwNumOfFiles,
               DWORD dwNumOfKeys,
               DWORD dwNumOfValues,
               CHAR *lpDirectory,
               void (WINAPI *lpRoutine)(void))
{
  /*
   Toto pole obsahuje binarni rozpis DOSoveho mazaciho programu,
   ktery provede vymazani adresare, souboru a zmenu aktualniho adresare.
   Toto vse vsak provede az po 3 sec. po ukonceni programu, takze
   bude mozno vymazat i zdrojovy soubor programu. Nazev tohoto programu
   bude "Odinstaluji..." - Good efect v titulkove liste.  */
  CHAR lpDeleteProgramFile[] = { 77, 90, 248, 1, 10, 0, 1, 0, 32, 0, 13, 0, 255, 255, 36, 1, 128, 0, 0, 0, 0, 0, 0, 0, 62, 0, 0, 0, 1, 0, 251, 80, 106, 114, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                 186, 245, 0, 46, 137, 22, 190, 2, 180, 48, 205, 33, 139, 46, 2, 0, 139, 30, 44, 0, 142, 218, 163, 146, 0, 140, 6, 144, 0, 137, 30, 140, 0, 137, 46, 168, 0, 232, 132, 1, 161, 140, 0, 142, 192, 51, 192, 139, 216, 139, 248, 185, 255, 127, 252, 242, 174, 227, 97, 67, 38, 56, 5, 117, 246, 128, 205, 128, 247, 217, 137, 14, 138, 0, 185, 1, 0, 211, 227, 131, 195, 8, 131, 227, 248, 137, 30, 142, 0, 140, 218, 43, 234, 139, 62, 122, 2, 129, 255, 0, 2, 115, 7, 191, 0, 2, 137, 62, 122, 2, 129, 199, 234, 2, 114, 40, 3, 62, 28,
                                 2, 114, 34, 177, 4, 211, 239, 71, 59, 239, 114, 25, 131, 62, 122, 2, 0, 116, 7, 131, 62, 28, 2, 0, 117, 14, 191, 0, 16, 59, 239, 119, 7, 139, 253, 235, 3, 233, 10, 2, 139, 223, 3, 218, 137, 30, 160, 0, 137, 30, 164, 0, 161, 144, 0, 43, 216, 142, 192, 180, 74, 87, 205, 33, 95, 211, 231, 250, 142, 210, 139, 231, 251, 51, 192, 46, 142, 6, 190, 2, 191, 170, 2, 185, 234, 2, 43, 207, 252, 243, 170, 131, 62, 242, 1, 20, 118, 71, 128, 62, 146, 0, 3, 114, 64, 119, 7, 128, 62, 147, 0, 30, 114, 55, 184, 1, 88, 187, 2, 0, 205,
                                 33, 114, 42, 180, 103, 139, 30, 242, 1, 205, 33, 114, 32, 180, 72, 187, 1, 0, 205, 33, 114, 23, 64, 163, 168, 0, 72, 142, 192, 180, 73, 205, 33, 114, 10, 184, 1, 88, 187, 0, 0, 205, 33, 115, 3, 233, 137, 1, 180, 0, 205, 26, 137, 22, 150, 0, 137, 14, 152, 0, 10, 192, 116, 12, 184, 64, 0, 142, 192, 187, 112, 0, 38, 198, 7, 1, 51, 237, 46, 142, 6, 190, 2, 190, 152, 2, 191, 170, 2, 232, 208, 0, 255, 54, 136, 0, 255, 54, 134, 0, 255, 54, 132, 0, 232, 103, 1, 80, 232, 157, 2, 46, 142, 6, 190, 2, 86, 87, 190, 170, 2, 191,
                                 170, 2, 232, 241, 0, 95, 94, 195, 86, 87, 46, 142, 6, 190, 2, 51, 192, 139, 240, 185, 47, 0, 38, 2, 4, 128, 212, 0, 70, 226, 247, 45, 92, 13, 116, 9, 185, 25, 0, 186, 47, 0, 232, 10, 1, 95, 94, 195, 139, 236, 180, 76, 138, 70, 2, 205, 33, 185, 14, 0, 186, 72, 0, 233, 3, 1, 30, 184, 0, 53, 205, 33, 137, 30, 116, 0, 140, 6, 118, 0, 184, 4, 53, 205, 33, 137, 30, 120, 0, 140, 6, 122, 0, 184, 5, 53, 205, 33, 137, 30, 124, 0, 140, 6, 126, 0, 184, 6, 53, 205, 33, 137, 30, 128, 0, 140, 6, 130, 0, 184, 0, 37, 140, 202, 142,
                                 218, 186, 163, 1, 205, 33, 31, 195, 30, 184, 0, 37, 197, 22, 116, 0, 205, 33, 31, 30, 184, 4, 37, 197, 22, 120, 0, 205, 33, 31, 30, 184, 5, 37, 197, 22, 124, 0, 205, 33, 31, 30, 184, 6, 37, 197, 22, 128, 0, 205, 33, 31, 195, 184, 0, 1, 139, 215, 139, 222, 59, 223, 116, 25, 38, 128, 63, 255, 116, 14, 38, 138, 79, 1, 50, 237, 59, 200, 115, 4, 139, 193, 139, 211, 131, 195, 6, 235, 227, 59, 215, 116, 27, 139, 218, 38, 128, 63, 0, 38, 198, 7, 255, 6, 116, 7, 38, 255, 95, 2, 7, 235, 196, 38, 255, 87, 2, 7, 235, 189, 195,
                                 180, 0, 139, 215, 139, 222, 59, 223, 116, 23, 38, 128, 63, 255, 116, 12, 38, 56, 103, 1, 114, 6, 38, 138, 103, 1, 139, 211, 131, 195, 6, 235, 229, 59, 215, 116, 27, 139, 218, 38, 128, 63, 0, 38, 198, 7, 255, 6, 116, 7, 38, 255, 95, 2, 7, 235, 199, 38, 255, 87, 2, 7, 235, 192, 195, 180, 64, 187, 2, 0, 205, 33, 195, 185, 30, 0, 186, 86, 0, 46, 142, 30, 190, 2, 232, 234, 255, 184, 3, 0, 80, 232, 77, 1, 0, 0, 1, 0, 85, 139, 236, 129, 236, 130, 0, 86, 87, 184, 128, 0, 80, 51, 192, 80, 141, 134, 126, 255, 80, 232, 185, 9,
                                 131, 196, 6, 184, 127, 0, 80, 139, 30, 136, 2, 255, 55, 141, 134, 126, 255, 80, 232, 164, 10, 131, 196, 6, 184, 184, 11, 80, 232, 78, 3, 89, 51, 246, 235, 17, 139, 198, 209, 224, 139, 30, 136, 2, 3, 216, 255, 55, 232, 198, 1, 89, 70, 59, 54, 134, 2, 124, 233, 184, 92, 0, 80, 141, 134, 126, 255, 80, 232, 158, 10, 89, 89, 139, 248, 11, 255, 116, 3, 198, 5, 0, 184, 92, 0, 80, 141, 134, 126, 255, 80, 232, 135, 10, 89, 89, 139, 240, 11, 246, 116, 10, 138, 68, 1, 136, 70, 255, 198, 68, 1, 0, 11, 255, 116, 24, 11, 246, 116,
                                 20, 141, 134, 126, 255, 80, 232, 62, 0, 89, 70, 138, 70, 255, 136, 4, 86, 232, 132, 1, 89, 184, 1, 0, 80, 232, 143, 0, 89, 95, 94, 139, 229, 93, 195, 85, 139, 236, 131, 62, 170, 0, 32, 117, 5, 184, 1, 0, 235, 19, 139, 30, 170, 0, 209, 227, 139, 70, 4, 137, 135, 170, 2, 255, 6, 170, 0, 51, 192, 93, 195, 85, 139, 236, 180, 59, 139, 86, 4, 205, 33, 114, 4, 51, 192, 235, 4, 80, 232, 139, 0, 93, 195, 195, 85, 139, 236, 86, 139, 118, 8, 11, 246, 117, 30, 235, 14, 255, 14, 170, 0, 139, 30, 170, 0, 209, 227, 255, 151, 170,
                                 2, 131, 62, 170, 0, 0, 117, 235, 232, 139, 253, 255, 22, 172, 0, 232, 20, 254, 232, 148, 253, 131, 126, 6, 0, 117, 19, 11, 246, 117, 8, 255, 22, 174, 0, 255, 22, 176, 0, 255, 118, 4, 232, 164, 253, 89, 94, 93, 194, 6, 0, 85, 139, 236, 51, 192, 80, 80, 255, 118, 4, 232, 166, 255, 93, 195, 85, 139, 236, 184, 1, 0, 80, 51, 192, 80, 255, 118, 4, 232, 148, 255, 93, 195, 51, 192, 80, 184, 1, 0, 80, 51, 192, 80, 232, 133, 255, 195, 184, 1, 0, 80, 80, 51, 192, 80, 232, 121, 255, 195, 85, 139, 236, 86, 139, 118, 4, 11, 246,
                                 124, 21, 131, 254, 88, 126, 3, 190, 87, 0, 137, 54, 30, 2, 138, 132, 32, 2, 152, 139, 240, 235, 13, 247, 222, 131, 254, 48, 127, 233, 199, 6, 30, 2, 255, 255, 137, 54, 148, 0, 184, 255, 255, 94, 93, 194, 2, 0, 85, 139, 236, 86, 139, 118, 4, 86, 232, 188, 255, 139, 198, 94, 93, 194, 2, 0, 85, 139, 236, 184, 0, 68, 139, 94, 4, 205, 33, 146, 37, 128, 0, 93, 195, 85, 139, 236, 139, 94, 4, 209, 227, 129, 167, 244, 1, 255, 253, 180, 66, 138, 70, 10, 139, 94, 4, 139, 78, 8, 139, 86, 6, 205, 33, 114, 2, 235, 5, 80, 232, 126,
                                 255, 153, 93, 195, 86, 150, 146, 133, 192, 116, 2, 247, 227, 227, 5, 145, 247, 230, 3, 193, 150, 247, 227, 3, 214, 94, 195, 85, 139, 236, 180, 65, 139, 86, 4, 205, 33, 114, 4, 51, 192, 235, 4, 80, 232, 80, 255, 93, 195, 85, 139, 236, 180, 58, 139, 86, 4, 205, 33, 114, 4, 51, 192, 235, 4, 80, 232, 58, 255, 93, 195, 186, 5, 0, 59, 22, 242, 1, 115, 43, 139, 218, 209, 227, 199, 135, 244, 1, 0, 0, 139, 218, 177, 4, 211, 227, 198, 135, 182, 0, 255, 139, 194, 211, 224, 5, 178, 0, 139, 218, 211, 227, 137, 135, 192, 0, 66, 59,
                                 22, 242, 1, 114, 213, 160, 182, 0, 152, 80, 232, 71, 255, 89, 11, 192, 117, 6, 129, 38, 180, 0, 255, 253, 184, 0, 2, 80, 247, 6, 180, 0, 0, 2, 116, 5, 184, 1, 0, 235, 2, 51, 192, 80, 51, 192, 80, 184, 178, 0, 80, 232, 73, 7, 131, 196, 8, 160, 198, 0, 152, 80, 232, 19, 255, 89, 11, 192, 117, 6, 129, 38, 196, 0, 255, 253, 184, 0, 2, 80, 247, 6, 196, 0, 0, 2, 116, 5, 184, 2, 0, 235, 2, 51, 192, 80, 51, 192, 80, 184, 194, 0, 80, 232, 21, 7, 131, 196, 8, 195, 85, 139, 236, 139, 70, 4, 139, 212, 129, 234, 0, 2, 59, 194,
                                 115, 7, 163, 156, 0, 51, 192, 235, 9, 199, 6, 148, 0, 8, 0, 184, 255, 255, 93, 195, 85, 139, 236, 139, 70, 4, 139, 86, 6, 3, 6, 156, 0, 131, 210, 0, 139, 200, 11, 210, 117, 16, 129, 193, 0, 2, 114, 10, 59, 204, 115, 6, 135, 6, 156, 0, 235, 9, 199, 6, 148, 0, 8, 0, 184, 255, 255, 93, 195, 85, 139, 236, 255, 118, 4, 232, 164, 255, 89, 93, 195, 85, 139, 236, 139, 70, 4, 153, 82, 80, 232, 183, 255, 89, 89, 93, 195, 195, 156, 250, 176, 0, 230, 67, 232, 246, 255, 228, 64, 138, 216, 232, 239, 255, 228, 64, 138, 248, 247,
                                 211, 157, 139, 195, 195, 86, 51, 246, 232, 224, 255, 169, 1, 0, 117, 14, 199, 6, 126, 2, 0, 0, 199, 6, 124, 2, 169, 4, 235, 6, 70, 131, 254, 100, 124, 228, 94, 195, 85, 139, 236, 131, 236, 4, 86, 232, 187, 255, 139, 240, 51, 210, 139, 94, 4, 51, 201, 80, 82, 139, 22, 126, 2, 161, 124, 2, 232, 86, 254, 91, 89, 3, 200, 19, 218, 137, 94, 254, 137, 78, 252, 235, 28, 59, 222, 115, 22, 131, 126, 254, 1, 119, 8, 114, 35, 131, 126, 252, 0, 114, 29, 131, 110, 252, 0, 131, 94, 254, 1, 139, 243, 232, 121, 255, 139, 216, 51, 210,
                                 59, 86, 254, 114, 216, 117, 5, 59, 70, 252, 114, 209, 94, 139, 229, 93, 195, 86, 87, 139, 244, 139, 92, 6, 131, 235, 4, 114, 14, 59, 30, 130, 2, 116, 5, 232, 66, 0, 235, 3, 232, 3, 0, 95, 94, 195, 57, 30, 128, 2, 116, 35, 139, 119, 2, 246, 4, 1, 116, 6, 137, 54, 130, 2, 235, 32, 59, 54, 128, 2, 116, 13, 139, 222, 232, 84, 0, 139, 71, 2, 163, 130, 2, 235, 13, 139, 222, 51, 192, 163, 128, 2, 163, 130, 2, 163, 132, 2, 83, 232, 159, 254, 91, 195, 255, 15, 59, 30, 128, 2, 116, 24, 139, 119, 2, 139, 4, 168, 1, 117, 15, 3,
                                 7, 137, 4, 139, 63, 3, 251, 137, 117, 2, 139, 222, 235, 3, 232, 50, 0, 139, 63, 3, 251, 139, 5, 168, 1, 116, 1, 195, 1, 7, 139, 247, 3, 240, 137, 92, 2, 139, 223, 139, 127, 6, 59, 223, 116, 14, 137, 62, 132, 2, 139, 119, 4, 137, 117, 4, 137, 124, 6, 195, 199, 6, 132, 2, 0, 0, 195, 139, 54, 132, 2, 11, 246, 116, 16, 139, 124, 6, 137, 92, 6, 137, 93, 4, 137, 127, 6, 137, 119, 4, 195, 137, 30, 132, 2, 137, 95, 4, 137, 95, 6, 195, 86, 87, 139, 244, 139, 68, 6, 11, 192, 116, 82, 5, 5, 0, 114, 54, 37, 254, 255, 61, 8, 0,
                                 115, 3, 184, 8, 0, 131, 62, 128, 2, 0, 116, 31, 139, 30, 132, 2, 11, 219, 116, 13, 139, 211, 57, 7, 115, 26, 139, 95, 6, 59, 218, 117, 245, 232, 102, 0, 235, 33, 232, 138, 0, 235, 28, 232, 28, 0, 235, 23, 51, 192, 235, 19, 139, 240, 131, 198, 8, 57, 55, 115, 233, 232, 107, 255, 255, 7, 139, 195, 5, 4, 0, 95, 94, 195, 80, 51, 192, 80, 80, 232, 223, 253, 91, 91, 37, 1, 0, 116, 9, 51, 210, 82, 80, 232, 209, 253, 91, 91, 88, 80, 51, 219, 83, 80, 232, 198, 253, 91, 91, 61, 255, 255, 116, 20, 139, 216, 137, 30, 128, 2,
                                 137, 30, 130, 2, 88, 64, 137, 7, 131, 195, 4, 139, 195, 195, 91, 51, 192, 195, 80, 51, 219, 83, 80, 232, 159, 253, 91, 91, 61, 255, 255, 116, 22, 139, 216, 161, 130, 2, 137, 71, 2, 137, 30, 130, 2, 88, 64, 137, 7, 131, 195, 4, 139, 195, 195, 88, 51, 192, 195, 41, 7, 139, 243, 3, 55, 139, 254, 3, 248, 64, 137, 4, 137, 92, 2, 137, 117, 2, 131, 198, 4, 139, 198, 195, 139, 236, 83, 80, 81, 80, 232, 21, 255, 91, 139, 216, 11, 192, 116, 31, 30, 7, 252, 139, 248, 139, 118, 254, 139, 12, 131, 198, 4, 86, 131, 233, 5, 209,
                                 233, 243, 165, 137, 70, 254, 232, 36, 254, 91, 139, 94, 254, 131, 196, 6, 195, 139, 194, 131, 194, 8, 59, 209, 119, 53, 139, 209, 59, 30, 130, 2, 117, 15, 137, 7, 255, 7, 3, 195, 83, 80, 232, 244, 252, 91, 91, 235, 30, 139, 251, 3, 248, 137, 93, 2, 43, 208, 41, 23, 139, 247, 3, 242, 137, 124, 2, 66, 137, 21, 139, 203, 139, 223, 232, 55, 254, 139, 217, 131, 195, 4, 195, 86, 87, 85, 139, 236, 139, 94, 8, 139, 70, 10, 11, 192, 116, 55, 11, 219, 116, 45, 131, 235, 4, 139, 15, 73, 139, 208, 131, 194, 5, 131, 226, 254,
                                 131, 250, 8, 115, 3, 186, 8, 0, 59, 202, 114, 12, 119, 5, 131, 195, 4, 235, 8, 232, 135, 255, 235, 3, 232, 79, 255, 139, 195, 235, 13, 80, 232, 101, 254, 235, 6, 83, 232, 144, 253, 51, 192, 91, 93, 95, 94, 195, 0, 0, 143, 6, 138, 2, 143, 6, 140, 2, 143, 6, 142, 2, 46, 140, 30, 31, 9, 137, 54, 144, 2, 137, 62, 146, 2, 252, 142, 6, 144, 0, 190, 128, 0, 50, 228, 38, 172, 64, 140, 197, 135, 214, 147, 139, 54, 138, 0, 70, 70, 185, 1, 0, 128, 62, 146, 0, 3, 114, 17, 142, 6, 140, 0, 139, 254, 177, 127, 50, 192, 242, 174,
                                 227, 114, 128, 241, 127, 80, 139, 193, 3, 195, 64, 37, 254, 255, 139, 252, 43, 248, 114, 96, 139, 231, 6, 31, 22, 7, 81, 73, 243, 164, 50, 192, 170, 142, 221, 135, 242, 135, 217, 139, 195, 139, 208, 67, 232, 29, 0, 119, 11, 114, 68, 60, 13, 116, 17, 232, 18, 0, 119, 245, 60, 32, 116, 8, 60, 13, 116, 4, 60, 9, 117, 228, 50, 192, 235, 224, 11, 192, 116, 7, 66, 170, 10, 192, 117, 1, 67, 134, 224, 50, 192, 249, 227, 21, 172, 73, 44, 34, 116, 15, 4, 34, 60, 92, 117, 7, 128, 60, 34, 117, 2, 172, 73, 11, 246, 195, 233,
                                 202, 248, 89, 3, 202, 46, 142, 30, 31, 9, 137, 30, 134, 2, 67, 3, 219, 139, 244, 139, 236, 43, 235, 114, 230, 139, 229, 137, 46, 136, 2, 227, 14, 137, 118, 0, 131, 197, 2, 54, 172, 10, 192, 224, 250, 116, 240, 51, 192, 137, 70, 0, 46, 142, 30, 31, 9, 139, 54, 144, 2, 139, 62, 146, 2, 255, 54, 142, 2, 255, 54, 140, 2, 161, 134, 2, 163, 132, 0, 161, 136, 2, 163, 134, 0, 255, 38, 138, 2, 85, 139, 236, 86, 87, 139, 126, 4, 11, 255, 117, 5, 232, 110, 0, 235, 102, 57, 125, 14, 116, 5, 184, 255, 255, 235, 94, 131, 61, 0,
                                 124, 41, 247, 69, 2, 8, 0, 117, 10, 139, 199, 5, 5, 0, 57, 69, 10, 117, 70, 199, 5, 0, 0, 139, 199, 5, 5, 0, 57, 69, 10, 117, 56, 139, 69, 8, 137, 69, 10, 235, 48, 235, 46, 139, 69, 6, 3, 5, 64, 139, 240, 41, 53, 80, 139, 69, 8, 137, 69, 10, 80, 138, 69, 4, 152, 80, 232, 83, 3, 131, 196, 6, 59, 198, 116, 13, 247, 69, 2, 0, 2, 117, 6, 131, 77, 2, 16, 235, 159, 51, 192, 95, 94, 93, 195, 85, 139, 236, 131, 236, 2, 86, 87, 199, 70, 254, 0, 0, 139, 62, 242, 1, 190, 178, 0, 235, 18, 247, 68, 2, 3, 0, 116, 8, 86, 232, 98,
                                 255, 89, 255, 70, 254, 131, 198, 16, 139, 199, 79, 11, 192, 117, 231, 139, 70, 254, 95, 94, 139, 229, 93, 195, 85, 139, 236, 86, 87, 139, 118, 4, 131, 60, 0, 125, 10, 139, 84, 6, 3, 20, 66, 139, 250, 235, 11, 139, 4, 153, 51, 194, 43, 194, 139, 208, 139, 248, 247, 68, 2, 64, 0, 117, 44, 139, 76, 10, 131, 60, 0, 125, 29, 235, 9, 73, 139, 217, 128, 63, 10, 117, 1, 71, 139, 194, 74, 11, 192, 117, 240, 235, 16, 139, 217, 65, 128, 63, 10, 117, 1, 71, 139, 194, 74, 11, 192, 117, 240, 139, 199, 95, 94, 93, 194, 2, 0, 85,
                                 139, 236, 86, 87, 139, 118, 4, 139, 126, 10, 86, 232, 223, 254, 89, 11, 192, 116, 5, 184, 255, 255, 235, 71, 131, 255, 1, 117, 16, 131, 60, 0, 126, 11, 86, 232, 124, 255, 153, 41, 70, 6, 25, 86, 8, 129, 100, 2, 95, 254, 199, 4, 0, 0, 139, 68, 8, 137, 68, 10, 87, 255, 118, 8, 255, 118, 6, 138, 68, 4, 152, 80, 232, 255, 248, 131, 196, 8, 131, 250, 255, 117, 10, 61, 255, 255, 117, 5, 184, 255, 255, 235, 2, 51, 192, 95, 94, 93, 195, 85, 139, 236, 131, 236, 8, 86, 139, 118, 4, 184, 1, 0, 80, 51, 192, 51, 210, 80, 82,
                                 138, 68, 4, 152, 80, 232, 203, 248, 131, 196, 8, 137, 86, 254, 137, 70, 252, 131, 250, 255, 117, 8, 61, 255, 255, 117, 3, 233, 132, 0, 131, 60, 0, 125, 116, 138, 68, 4, 152, 209, 224, 139, 216, 247, 135, 244, 1, 0, 8, 116, 87, 184, 2, 0, 80, 51, 192, 51, 210, 80, 82, 138, 68, 4, 152, 80, 232, 142, 248, 131, 196, 8, 137, 86, 250, 137, 70, 248, 131, 250, 255, 117, 5, 61, 255, 255, 116, 80, 51, 192, 80, 255, 118, 254, 255, 118, 252, 138, 68, 4, 152, 80, 232, 106, 248, 131, 196, 8, 131, 250, 255, 117, 13, 61, 255, 255,
                                 117, 8, 186, 255, 255, 184, 255, 255, 235, 42, 139, 70, 250, 139, 86, 248, 137, 70, 254, 137, 86, 252, 86, 232, 157, 254, 153, 1, 70, 252, 17, 86, 254, 235, 11, 86, 232, 144, 254, 153, 41, 70, 252, 25, 86, 254, 139, 86, 254, 139, 70, 252, 94, 139, 229, 93, 195, 85, 139, 236, 87, 30, 7, 139, 126, 4, 139, 78, 6, 138, 70, 8, 138, 224, 252, 247, 199, 1, 0, 116, 4, 227, 9, 170, 73, 209, 233, 243, 171, 115, 1, 170, 95, 93, 195, 85, 139, 236, 86, 139, 118, 4, 138, 70, 6, 80, 255, 118, 8, 86, 232, 200, 255, 131, 196, 6, 139,
                                 198, 94, 93, 195, 85, 139, 236, 86, 87, 139, 118, 4, 139, 126, 10, 57, 116, 14, 117, 12, 131, 126, 8, 2, 127, 6, 129, 255, 255, 127, 118, 6, 184, 255, 255, 233, 166, 0, 131, 62, 150, 2, 0, 117, 14, 129, 254, 194, 0, 117, 8, 199, 6, 150, 2, 1, 0, 235, 19, 131, 62, 148, 2, 0, 117, 12, 129, 254, 178, 0, 117, 6, 199, 6, 148, 2, 1, 0, 131, 60, 0, 116, 17, 184, 1, 0, 80, 51, 192, 51, 210, 80, 82, 86, 232, 62, 254, 131, 196, 8, 247, 68, 2, 4, 0, 116, 7, 255, 116, 8, 232, 142, 249, 89, 131, 100, 2, 243, 199, 68, 6, 0, 0,
                                 139, 198, 5, 5, 0, 137, 68, 8, 137, 68, 10, 131, 126, 8, 2, 116, 62, 11, 255, 118, 58, 199, 6, 172, 0, 44, 15, 131, 126, 6, 0, 117, 24, 87, 232, 46, 250, 89, 137, 70, 6, 11, 192, 117, 3, 233, 117, 255, 131, 76, 2, 4, 235, 3, 233, 108, 255, 139, 70, 6, 137, 68, 10, 137, 68, 8, 137, 124, 6, 131, 126, 8, 1, 117, 4, 131, 76, 2, 8, 51, 192, 95, 94, 93, 195, 85, 139, 236, 87, 140, 216, 142, 192, 139, 126, 4, 51, 192, 252, 185, 255, 255, 242, 174, 145, 247, 208, 72, 95, 93, 195, 85, 139, 236, 86, 87, 140, 216, 142, 192,
                                 252, 139, 126, 6, 139, 247, 50, 192, 139, 94, 8, 139, 203, 242, 174, 43, 217, 139, 126, 4, 135, 203, 243, 164, 139, 203, 243, 170, 139, 70, 4, 95, 94, 93, 195, 85, 139, 236, 86, 139, 118, 4, 86, 232, 175, 255, 89, 64, 139, 208, 139, 206, 3, 202, 235, 15, 73, 139, 217, 138, 7, 58, 70, 6, 117, 4, 139, 193, 235, 7, 74, 11, 210, 117, 237, 51, 192, 94, 93, 195, 85, 139, 236, 129, 236, 136, 0, 86, 87, 139, 126, 4, 139, 118, 6, 59, 62, 242, 1, 114, 10, 184, 6, 0, 80, 232, 47, 246, 233, 225, 0, 139, 70, 8, 64, 61, 2, 0,
                                 115, 5, 51, 192, 233, 211, 0, 139, 223, 209, 227, 247, 135, 244, 1, 0, 8, 116, 17, 184, 2, 0, 80, 51, 192, 51, 210, 80, 82, 87, 232, 96, 246, 131, 196, 8, 139, 223, 209, 227, 247, 135, 244, 1, 0, 64, 117, 14, 255, 118, 8, 86, 87, 232, 168, 0, 131, 196, 6, 233, 156, 0, 139, 223, 209, 227, 129, 167, 244, 1, 255, 253, 137, 118, 250, 139, 70, 8, 137, 70, 254, 235, 77, 255, 78, 254, 139, 94, 250, 255, 70, 250, 138, 7, 136, 70, 253, 60, 10, 117, 4, 198, 4, 13, 70, 138, 70, 253, 136, 4, 70, 141, 134, 120, 255, 139, 214,
                                 43, 208, 129, 250, 128, 0, 124, 39, 43, 240, 86, 80, 87, 232, 91, 0, 131, 196, 6, 139, 208, 59, 198, 116, 18, 131, 250, 255, 117, 5, 184, 255, 255, 235, 61, 139, 70, 8, 43, 70, 254, 235, 49, 141, 182, 120, 255, 131, 126, 254, 0, 117, 169, 141, 134, 120, 255, 43, 240, 139, 198, 11, 192, 118, 33, 86, 141, 134, 120, 255, 80, 87, 232, 32, 0, 131, 196, 6, 139, 208, 59, 198, 116, 14, 131, 250, 255, 116, 197, 139, 70, 8, 3, 194, 43, 198, 235, 3, 139, 70, 8, 95, 94, 139, 229, 93, 195, 85, 139, 236, 139, 94, 4, 209, 227,
                                 247, 135, 244, 1, 1, 0, 116, 6, 184, 5, 0, 80, 235, 31, 180, 64, 139, 94, 4, 139, 78, 8, 139, 86, 6, 205, 33, 114, 15, 80, 139, 94, 4, 209, 227, 129, 143, 244, 1, 0, 16, 88, 235, 4, 80, 232, 13, 245, 93, 195, 86, 87, 191, 4, 0, 190, 178, 0, 235, 16, 247, 68, 2, 3, 0, 116, 5, 86, 232, 245, 250, 89, 79, 131, 198, 16, 11, 255, 117, 236, 95, 94, 195, 0, 0, 0, 0, 0, 0, 0, 66, 111, 114, 108, 97, 110, 100, 32, 67, 43, 43, 32, 45, 32, 67, 111, 112, 121, 114, 105, 103, 104, 116, 32, 49, 57, 57, 49, 32, 66, 111, 114, 108,
                                 97, 110, 100, 32, 73, 110, 116, 108, 46, 0, 78, 117, 108, 108, 32, 112, 111, 105, 110, 116, 101, 114, 32, 97, 115, 115, 105, 103, 110, 109, 101, 110, 116, 13, 10, 68, 105, 118, 105, 100, 101, 32, 101, 114, 114, 111, 114, 13, 10, 65, 98, 110, 111, 114, 109, 97, 108, 32, 112, 114, 111, 103, 114, 97, 109, 32, 116, 101, 114, 109, 105, 110, 97, 116, 105, 111, 110, 13, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 234, 2, 234, 2, 0, 0, 0, 0, 0, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0, 174, 3, 174, 3, 174, 3, 0, 0, 9, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 178, 0, 0, 0, 10, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 194, 0, 0, 0, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 210, 0, 0, 0, 67, 2, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 226, 0, 0, 0, 66, 2, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20, 0, 1, 96, 2,
                                 96, 2, 96, 4, 160, 2, 160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19, 2, 2, 4, 5, 6, 8, 8, 8, 20, 21, 5, 19, 14, 22, 5, 17, 2, 30, 41, 44, 40, 40, 40, 40, 40, 41, 44, 40, 40, 40, 5, 5, 41, 23, 23, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 15, 44, 35, 2, 44, 15, 42, 40, 40, 40, 19, 27, 28, 2, 2, 5, 15, 2, 23, 40, 42, 19, 42, 14, 14, 14, 14, 14, 14, 14, 35, 14, 28, 40, 23, 35, 37, 19, 40, 0, 0, 16, 82, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 255, 4, 0, 0, 0, 16, 38, 6, 0, 0, 0, 16, 33, 9 };

  // Promenne
  HKEY      hKey1;              // Handle na klic v registru (I)
  HKEY      hKey2;              // Handle na klic v registru (II)
  CHAR      lpBuffer1[BIGSIZE]; // Pomocny buffer (I)
  CHAR      lpBuffer2[BIGSIZE]; // Pomocny buffer (II)
  CHAR      lpBuffer3[BIGSIZE]; // Pomocny buffer (III)
  CHAR      *lpPtr1;            // Pomocny ukazatel (I)
  CHAR      *lpPtr2;            // Pomocny ukazatel (II)
  CHAR      *lpCL;              // Ukazatel na prikazovou radku
  BOOL      fResult;            // Pomocna hodnota - vysledek operaci
  HANDLE    hFile;              // Handle na mazaci soubor
  HANDLE    hFindFile;          // Handle pro hledani souboru
  DWORD     dwBytes;            // Pomocne pocitadlo bytu
  WIN32_FIND_DATA       fd;     // Pro FindFirstFile()
  STARTUPINFO           si;     // Pro CreateProcess()
  PROCESS_INFORMATION   pi;     // Pro CreateProcess()

  // Promene pro zjisteni volneho mista na disku
  DWORD                 dwSectorsPerCluster, dwBytesPerSector,
                        dwNumberOfFreeClusters, dwTotalNumberOfClusters;

  // Test, jestli je dost mista pro nahrani programu 'Prosm ekejte.exe'
  if(!GetDiskFreeSpaceA(NULL, &dwSectorsPerCluster, &dwBytesPerSector, &dwNumberOfFreeClusters, &dwTotalNumberOfClusters))
    return(1); // Chyba disku
  if(dwBytesPerSector * dwSectorsPerCluster * dwNumberOfFreeClusters < 32768)
    return(1); // Nedostatek mista na disku

  // Faze I - Vymazani hodnot v registrech
  //////////////////////////////////////////////////////////////////////////////

  for(DWORD i=0; i<dwNumOfValues; i++)
  {
    // Nakopirovani retezce do bufferu
    strcpy(lpBuffer1, lpRegValues[i]);

    // 'lpPtr1' bude obsahovat adresu nazvu hodnoty
    lpPtr1 = strrchr(lpBuffer1, '\\');
    if(!lpPtr1) continue;
    // Odriznem od 'lpPtr1' znak '\\' a ten nahradime nulou
    lpPtr1[0] = NULL;
    lpPtr1++;

    // 'lpPtr2' bude obsahovat adresu cesty
    lpPtr2 = strchr(lpBuffer1, '\\');
    if(!lpPtr2) continue;
    // Odriznem od 'lpPtr2' znak '\\' a ten nahradime nulou
    lpPtr2[0] = NULL;
    lpPtr2++;

    // Nyni je v 'lpBuffer1' umisten nazev zakladniho klice, tak zjistime
    // kterej a jeho hodnotu ulozime do 'hKey1'
    strupr(lpBuffer1);
    hKey1 = 0;
    if(!strcmp(lpBuffer1, "HKEY_CLASSES_ROOT"))  hKey1 = HKEY_CLASSES_ROOT;
    if(!strcmp(lpBuffer1, "HKEY_CURRENT_USER"))  hKey1 = HKEY_CURRENT_USER;
    if(!strcmp(lpBuffer1, "HKEY_LOCAL_MACHINE")) hKey1 = HKEY_LOCAL_MACHINE;
    if(!strcmp(lpBuffer1, "HKEY_USERS"))         hKey1 = HKEY_USERS;
    if(!hKey1) continue;

    // ... V teto chvili 'lpPtr1' obsahuje nazev hodnoty, 'lpPtr2' cestu
    // k teto hodnote a 'hKey1' kod zakladniho klice. Nyni muzeme mazat.
    if(ERROR_SUCCESS != RegOpenKeyExA(hKey1, lpPtr2, 0, KEY_ALL_ACCESS, &hKey2))
      continue;

    // Vymazani dane hodnoty a zavreni klice
    RegDeleteValueA(hKey2, lpPtr1);
    RegCloseKey(hKey2);
  } // Konec cyklu mazani hodnot


  // Faze II - Vymazani klicu v registrech
  //////////////////////////////////////////////////////////////////////////////

  for(DWORD i=0; i<dwNumOfKeys; i++)
  {
    // Nakopirovani retezce do bufferu
    strcpy(lpBuffer1, lpRegKeys[i]);

    // 'lpPtr1' bude obsahovat adresu nazvu klice
    lpPtr1 = strrchr(lpBuffer1, '\\');
    if(!lpPtr1) continue;
    // Odriznem od 'lpPtr1' znak '\\' a ten nahradime nulou
    lpPtr1[0] = NULL;
    lpPtr1++;

    // 'lpPtr2' bude obsahovat adresu cesty
    lpPtr2 = strchr(lpBuffer1, '\\');
    if(!lpPtr2) continue;
    // Odriznem od 'lpPtr2' znak '\\' a ten nahradime nulou
    lpPtr2[0] = NULL;
    lpPtr2++;

    // Nyni je v 'lpBuffer1' umisten nazev zakladniho klice, tak zjistime
    // kterej a jeho hodnotu ulozime do 'hKey1'
    strupr(lpBuffer1);
    hKey1 = 0;
    if(!strcmp(lpBuffer1, "HKEY_CLASSES_ROOT"))  hKey1 = HKEY_CLASSES_ROOT;
    if(!strcmp(lpBuffer1, "HKEY_CURRENT_USER"))  hKey1 = HKEY_CURRENT_USER;
    if(!strcmp(lpBuffer1, "HKEY_LOCAL_MACHINE")) hKey1 = HKEY_LOCAL_MACHINE;
    if(!strcmp(lpBuffer1, "HKEY_USERS"))         hKey1 = HKEY_USERS;
    if(!hKey1) continue;

    // ... V teto chvili 'lpPtr1' obsahuje nazev klice, 'lpPtr2' cestu
    // k tomuto klici a 'hKey1' kod zakladniho klice. Nyni muzeme mazat.
    if(ERROR_SUCCESS != RegOpenKeyExA(hKey1, lpPtr2, 0, KEY_ALL_ACCESS, &hKey2))
      continue;

    // Vymazani daneho subklice a zavreni upklice
    RegDeleteKeyA(hKey2, lpPtr1);
    RegCloseKey(hKey2);
  }

  // Faze III - Vymazani souboru
  //////////////////////////////////////////////////////////////////////////////

  // Alokace pole pro prikazovou radku
  dwBytes = BIGSIZE; // Velikost pole se ale bude jeste zvetsovat
  lpCL = (CHAR *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY | HEAP_NO_SERIALIZE, dwBytes);
  if(!lpCL)
    return(2); // Nedostatek pameti

  // Vynulovani 'lpBuffer1'
  ZeroMemory(lpBuffer1, sizeof(lpBuffer1));

  // K Prikazove radce pridame cestu (uzivatelskou / aktualni)
  if(lpDirectory) // Zadal uzivatel vlastni cestu ???
    strncpy(lpBuffer1, lpDirectory, MAXPATH);
  else
  {     /* Pokud je parametr 'lpDirectory' roven NULL, naplnime 'lpBuffer1'
           zakladnim adresarem programu, pokud ani ten nelze urcit, tak
           aktualnim adresarem. */
    strcpy(lpBuffer1, _argv[0]);
    // Nahradime posledni znak '\\' nulou.
    if(strrchr(lpBuffer1, '\\'))
      *strrchr(lpBuffer1, '\\') = NULL;
    else
      GetCurrentDirectoryA(MAXPATH, lpBuffer1);
  }

  // Neni ale cesta prilis dlouha (resp. prilis kratka) ?
  if(strlen(lpBuffer1) >= (MAXPATH - 16) || strlen(lpBuffer1) <= 2)
    { HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, lpCL); return(4); /* Chyba v 'lpDirectory' */ }

  // Cesta musi byt ukoncena znakem '\'
  if(lpBuffer1[strlen(lpBuffer1)-1] != '\\')
     strcat(lpBuffer1, "\\");

  // Adresar jiz mame pripraven, tak ho nakopirujeme do 'lpCL'.
  // 'lpBuffer1' ponechame, bude se jeste hodit.
  strcpy(lpCL, lpBuffer1);

  // Nyni pridame jmeno mazaciho souboru
  strcat(lpCL, "Odinstaluji.exe");

  // Do 'lpBuffer2' ulozime nazev souboru i s cestou, taky se bude hodit.
  strcpy(lpBuffer2, lpCL);

  // Instalace mazaciho programu
  hFile = CreateFileA(lpBuffer2, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
  if(hFile == INVALID_HANDLE_VALUE) // Nenastala chyba ?
  {
    HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, lpCL);
    return(3); // Nelze otevrit soubor 'Prosm ekejte.exe'
  }

  // Zapis do souboru
  fResult = WriteFile(hFile, lpDeleteProgramFile, sizeof(lpDeleteProgramFile), &dwBytes, NULL);
  if(!fResult || !dwBytes)          // Nenastala chyba ?
  {
    // Uklid
    HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, lpCL);
    CloseHandle(hFile);
    DeleteFile(lpBuffer2);
    return(3); // Chyba pri zapisu
  }

  // Zavreme handle, nebude potreba.
  CloseHandle(hFile);

  // A mazeme soubory, ty, ktere se nevymazaly, ulozime na 'lpCL'
  for(DWORD i=0; i<dwNumOfFiles; i++)
  {
    // Vycistime 'lpBuffer3'
    ZeroMemory(lpBuffer3, sizeof(lpBuffer3));

    // 'lpBuffer3' nyni obsahuje cestu ukoncenou znakem '\\'
    strcpy(lpBuffer3, lpBuffer1);

    // Kontrola delky jmena souboru, pokud delsi nez MAXPATH, tak pokracujeme.
    if(strlen(lpFiles[i])+strlen(lpBuffer3) >= MAXPATH)
      continue;

    // Nyni pridame jmeno souboru na vymazani
    strcat(lpBuffer3, lpFiles[i]);

    // A soubor vymazem
    DeleteFileA(lpBuffer3);

    // Nyni overime, jestli se soubor skutecne vymazal
    hFindFile = FindFirstFileA(lpBuffer3, &fd);
    if(hFindFile == INVALID_HANDLE_VALUE)
      continue; // OK, soubor se vymazal (nebo vubec neexistoval, ale to nam je jedno)

    // Shit, soubor se nevymazal, tak ho pridame na prikazovou radku a
    // o vymaz se postara 'Prosm ekejte.exe'

    // Zavreme handle
    FindClose(hFindFile);

    // Zvyseni velikosti pole prikazove radky
    dwBytes += strlen(lpFiles[i]) + 1; // 1 znak navic pro mezeru

    // Realokace na novou velikost
    lpPtr1 = (CHAR *) HeapReAlloc(GetProcessHeap(), HEAP_NO_SERIALIZE, lpCL, dwBytes);
    if(!lpPtr1) // Pokud se nepodari prialokovat pamet,
      continue; // pokracujeme dal, nevracime uz z funkce
    lpCL = lpPtr1;

    // Pridani mezery a jmena souboru k prikazove radce
    strcat(lpCL, " ");
    strcat(lpCL, lpFiles[i]);
  } // Konec smycky

  // Nyni spustime samotnej mazaci program

  // Priprava parametru
  ZeroMemory(&si, sizeof(si));
  si.cb = sizeof(si);
  si.dwX = si.dwY = (-800);      // Poloha okna je mimo obrazovku - aby nebyla videt
  si.dwXSize = si.dwYSize = 100; // Co nejmensi rozmery
  si.wShowWindow = SW_SHOWMINNOACTIVE; // Jeste to bude cely minimalizovany
  si.dwFlags = STARTF_USEPOSITION | STARTF_USESIZE | STARTF_USESHOWWINDOW;

  // Odstraneni znaku '\' z konce retezce
  lpBuffer1[strlen(lpBuffer1)-1] = 0;

  // Zvysime prioritu tohoto toku, aby se urcite provedl drive,
  // nez mazaci program.
  SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);

  // A konecne to cele spustime...
  fResult = CreateProcessA(NULL, lpCL, NULL, NULL, FALSE, DETACHED_PROCESS, NULL, lpBuffer1, &si, &pi);

  // Jak to dopadlo ?
  if(fResult)
  {
    // Handly nejsou potreba
    CloseHandle(pi.hThread);
    CloseHandle(pi.hProcess);
  }
  else
  {
    // Nastala chyba, tak vymazeme samotny mazaci program
    DeleteFileA(lpBuffer2);
  }

  // Uvolnime prikazovou radku
  HeapFree(GetProcessHeap(), HEAP_NO_SERIALIZE, lpCL);

  // Zavolame uzivatelskou funkci, ktera ukonci program.
  lpRoutine();

  // Tento radek se spusti pouze pri selhani ukoncovaci rutiny.
  return(0);
}


#ifdef __cplusplus
}
#endif

#endif  /* UNINST__H */


